import $ from "jquery";
import DataTable from "datatables.net";
import HSDropdown from "@preline/dropdown";
import HSOverlay, { IOverlayOptions } from "@preline/overlay";
import HSSelect, { ISelectOptions } from "@preline/select";
import HSDataTable, { IDataTableOptions } from "@preline/datatable";
import { ConfigColumnDefs, AjaxSettings } from "datatables.net-dt";
import { autoUpdate, computePosition, offset } from "@floating-ui/dom";
import { formatTimestamp, verifyChannels } from "../../../src/ts/main";
import prisma from "../../../../../generated/prisma";

declare let guildId: string;
declare let channels: Array<any>;

// #region DataTable
//

// Fix dependency bugs with preline
(window as any).DataTable = DataTable;
(window as any).$hsDataTableCollection = [];

const emptyTableHtml: string = `
    <div class="max-w-md w-full min-h-[400px] flex flex-col justify-center mx-auto px-6 py-4">
        <div class="flex justify-center items-center size-[46px] bg-gray-100 rounded-lg dark:bg-neutral-800">
            <svg class="shrink-0 size-6 text-gray-600 dark:text-neutral-400" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24" stroke="currentColor" stroke-width="2" fill="none" stroke-linecap="round" stroke-linejoin="round"><path d="M4 11a9 9 0 0 1 9 9"></path><path d="M4 4a16 16 0 0 1 16 16"></path><circle cx="5" cy="19" r="1"></circle></svg>
        </div>
        <h2 class="mt-5 font-semibold text-gray-800 dark:text-white">
            No results found
        </h2>
        <p class="mt-2 text-sm text-gray-600 dark:text-neutral-400">
            Refine your search or create a new feed.
            Alternatively, use a template to deploy a ready-made feed.
        </p>

        <div class="mt-5 flex flex-col sm:flex-row gap-2">
            <button type="button" class="open-edit-modal-js py-2 px-3 inline-flex justify-center items-center gap-x-2 text-sm font-medium rounded-lg border border-transparent bg-blue-600 text-white hover:bg-blue-700 focus:outline-hidden focus:bg-blue-700 disabled:opacity-50 disabled:pointer-events-none" data-hs-overlay="#TODO">
                <svg class="shrink-0 size-4" xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"><path d="M5 12h14"/><path d="M12 5v14"/></svg>
                Create a feed
            </button>
            <button type="button" onclick="alert('not implemented');" class="py-2 px-3 inline-flex justify-center items-center gap-x-2 text-sm font-medium rounded-lg border border-gray-200 bg-white text-gray-800 shadow-xs hover:bg-gray-50 focus:outline-hidden focus:bg-gray-50 disabled:opacity-50 disabled:pointer-events-none dark:bg-neutral-800 dark:border-neutral-700 dark:text-white dark:hover:bg-neutral-700 dark:focus:bg-neutral-700">
                Use a Template
            </button>
        </div>
    </div>
`;

const columnDefs: ConfigColumnDefs[] = [
    // Select checkbox column
    {
        target: 0,
        orderable: false,
        searchable: false,
        className: "size-px whitespace-nowrap",
        render: (_data: unknown, _type: unknown, row: prisma.Feed) => { return `
            <div class="ps-6 py-4">
                <label class="rowSelect${row.id}-js" class="flex">
                    <input type="checkbox" id="rowSelect${row.id}-js" class="cj-table-checkbox" data-hs-datatable-row-selecting-individual=""/>
                    <span class="sr-only">Select Row</span>
                </label>
            </div>
        `}
    },
    {
        target: 1,
        data: "name",
        orderable: true,
        searchable: true,
        className: "size-px whitespace-nowrap",
        render: (data: string) => { return `
            <span class="cj-table-link max-w-[250px] truncate">
                ${data}
            </span>
        `}
    },
    {
        target: 2,
        data: "url",
        orderable: true,
        searchable: true,
        className: "size-px whitespace-nowrap",
        render: (data: string) => { return `
            <a href="${data}" class="cj-table-link max-w-[450px] truncate">
                ${data}
            </a>
        `}
    },
    {
        target: 3,
        data: "channels",
        orderable: false,
        searchable: false,
        className: "size-px",
        render: (data: prisma.Channel[], type: string, row: prisma.Feed) => {
            if (type !== "display") { return data; }
            if (!data.length) { return ""; }

            const wrapper = $("<div>").addClass("flex flex-nowrap gap-1 px-6 py-4");
            const tag = $("<span>").addClass("inline-flex items-center whitespace-nowrap gap-1 py-1 px-2.5 border text-xs rounded-md bg-white dark:bg-neutral-800 border-gray-200 dark:border-neutral-700 text-gray-800 dark:text-neutral-200");

            if (!verifyChannels(data, channels)) {
                wrapper.text("invalid channels").addClass("whitespace-nowrap");
                return wrapper.get(0);
            }

            const firstChannelName = "# " + channels.find(c => c.id === data[0].channel_id).name;
            wrapper.append(tag.clone().text(firstChannelName));

            // No need to run the dropdown code if there's no more to show
            if (data.length === 1) {
                return wrapper.get(0);
            }
            else if (data.length <= 2) {
                const secondChannelName = "# " + channels.find(c => c.id === data[1].channel_id).name;
                wrapper.append(tag.clone().text(secondChannelName));
                data.shift();
                return wrapper.get(0);
            }

            // drop the first element to exclude it from the dropdown
            data.shift();

            const dropdown = $("<div>").addClass("hs-dropdown inline-block");
            const dropdownBtn = $("<button>").attr("id", `channelDrop-${row.id}`).attr("type", "button").addClass("cursor-pointer inline-flex items-center gap-1 py-1 px-2.5 border text-xs rounded-md bg-white dark:bg-neutral-800 border-gray-200 dark:border-neutral-700 text-gray-800 dark:text-neutral-200");
            const dropdownMenu = $("<div>").addClass("hs-dropdown-menu hidden opacity-0 hs-dropdown-open:opacity-100 transition-[opacity,margin] overflow-hidden z-10 w-fit max-w-64 border p-2 rounded-md bg-gray-200 dark:bg-neutral-700 border-gray-300 dark:border-neutral-600");

            dropdown.append(dropdownBtn.text(`+${data.length}`));

            data.forEach(channel => {
                const channelName = "# " + channels.find(c => c.id === channel.channel_id).name;
                dropdownMenu.append(tag.clone().text(channelName));
            });

            dropdown.append(dropdownMenu);
            wrapper.append(dropdown);
            return wrapper.get(0);
        }
    },
    {
        target: 4,
        data: "filters",
        orderable: false,
        searchable: false,
        className: "size-px whitespace-nowrap",
        render: (data: prisma.Filter[]) => { return `
            <div class="px-6 py-4">
                <span class="cj-table-text">
                    ${data}
                </span>
            </div>
        `}
    },
    {
        target: 5,
        data: "message_style",
        orderable: false,  // both should be true, but message_style doesnt exist yet
        searchable: false,
        className: "size-px whitespace-nowrap",
        render: (data: string) => { return `
            <div class="px-6 py-4">
                <span class="cj-table-text">
                    ${data}
                </span>
            </div>
        `}
    },
    {
        target: 6,
        data: "created_at",
        orderable: true,
        searchable: false,
        className: "size-px whitespace-nowrap",
        render: (data: string) => { return `
            <div class="px-6 py-4">
                <span class="cj-table-text">
                    ${formatTimestamp(data)}
                </span>
            </div>
        `}
    },
    {
        target: 7,
        data: "active",
        orderable: true,
        searchable: false,
        className: "size-px whitespace-nowrap",
        render: (data: boolean) => {
            const wrapper = $("<div>").addClass("px-6 py-4");
            const badge = $("<span>").addClass("py-1 px-1.5 inline-flex items-center gap-x-1 text-xs font-medium rounded-full");
            const label = $("<span>");

            if (data) {
                badge.addClass("bg-teal-100 text-teal-800 dark:bg-teal-500/10 dark:text-teal-500");
                badge.append($('<svg class="size-2.5" xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" viewBox="0 0 16 16"><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-3.97-3.03a.75.75 0 0 0-1.08.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-.01-1.05z"/></svg>'))
                    .append(label.text("Active"));
            } else {
                badge.addClass("bg-red-100 text-red-800 dark:bg-red-500/10 dark:text-red-500");
                badge.append($('<svg class="size-2.5" xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" viewBox="0 0 16 16"><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.354 4.646a.5.5 0 1 0-.708.708L7.293 8l-2.647 2.646a.5.5 0 0 0 .708.708L8 8.707l2.646 2.647a.5.5 0 0 0 .708-.708L8.707 8l2.647-2.646a.5.5 0 0 0-.708-.708L8 7.293 5.354 4.646z"/></svg>'))
                    .append(label.text("Inactive"));
            }

            wrapper.append(badge);
            return wrapper.get(0);
        }
    }
];

const ajaxSettings: AjaxSettings = {
    url: `/guild/${guildId}/feeds/api/datatable`,
    type: "POST",
    contentType: "application/json",
    dataSrc: "data",
    data: (data: unknown) => {
        if (data === undefined) return;
        // TODO,
        return JSON.stringify(data);
    }
};

const tableOptions: IDataTableOptions = {
    ajax: ajaxSettings,
    serverSide: true,
    processing: true,
    columnDefs: columnDefs,
    pagingOptions: { pageBtnClasses: "hidden" },
    rowSelectingOptions: { selectAllSelector: "#selectAllBox" },
    language: {
        zeroRecords: emptyTableHtml,
        emptyTable: emptyTableHtml,
        loadingRecords: "Placeholder loading message..."
    },
    drawCallback: () => HSDropdown.autoInit(),
    rowCallback: (row: HTMLTableRowElement) => {
        $(row).addClass("bg-white dark:bg-neutral-900");
    }
};

const table: HSDataTable = new HSDataTable(
    $("#table").get(0) as HTMLElement,
    tableOptions
);

// #endregion

// #region Page Size Select
// https://preline.co/plugins/html/advanced-select.html

(window as any).$hsSelectCollection = [];
(window as any)["FloatingUIDOM"] = {
    computePosition: computePosition,
    autoUpdate: autoUpdate,
    offset: offset
};

const pageSelectOptions: ISelectOptions = {
    toggleTag: "<button type=\"button\" aria-expanded=\"false\"></button>",
    optionTemplate: `
        <div class=\"flex justify-between items-center w-full\">
            <span data-title></span>
            <span class=\"hidden hs-selected:block\">
                <svg class=\"shrink-0 size-3.5 text-blue-600 dark:text-blue-500\" xmlns=\"http:.w3.org/2000/svg\" width=\"24\" height=\"24\" viewBox=\"0 0 24 24\" fill=\"none\" stroke=\"currentColor\" stroke-width=\"2\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><polyline points=\"20 6 9 17 4 12\"/></svg>
            </span>
        </div>`,
    toggleClasses: "cj-select-toggle",
    optionClasses: "cj-select-option",
    dropdownClasses: `cj-select-dropdown`,
    dropdownSpace: 10,
    dropdownScope: "parent",
    dropdownPlacement: "top",
    dropdownVerticalFixedPlacement: null
};

const pageSizeSelect: HSSelect = new HSSelect(
    $("#selectPageSize-js").get(0) as HTMLElement,
    pageSelectOptions
);

// #endregion

// #region Edit Modal

(window as any).$hsOverlayCollection = [];

const editModalOptions: IOverlayOptions = {};

const editModal: HSOverlay = new HSOverlay(
    $("#editModal").get(0) as HTMLElement,
    editModalOptions
);

$(document).on("click", ".open-edit-modal-js", async () => {
    await openEditModal(-1);
});

const openEditModal = async (id: number) => {
    editModal.open();
};

const closeEditModal = () => {
    editModal.close();
};

// #endregion
